//#include "stdafx.h" // Unless using MFC, comment this line.

#include "ModuleGraph.h"

/*
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
*/

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CModuleGraph::CModuleGraph()
{
	m_iterModuleNode	= m_listModuleNode.end();
}

CModuleGraph::~CModuleGraph()
{
	Clear();
}

void CModuleGraph::Clear()
{
	CModuleNode*		pNode = NULL;
	MODULENODE_ITER		itEnd = m_listModuleNode.end();

	for ( MODULENODE_ITER it = m_listModuleNode.begin() ; it != itEnd ; ++it )
	{
		delete (*it);
	}
	m_listModuleNode.clear();
}


CModuleNode* CModuleGraph::FindNodeInList ( unsigned int uiId )
{
	CModuleNode*		pNode = NULL;
	MODULENODE_ITER		itEnd = m_listModuleNode.end();

	for ( MODULENODE_ITER it = m_listModuleNode.begin() ; it != itEnd ; ++it )
	{
		pNode = (*it);
		if ( pNode->m_pInfo->m_uiId == uiId )
		{
			return pNode;
		}
	}
	return NULL;
}

CModuleNode* CModuleGraph::HitTest ( int nX , int nY )
{
	CModuleNode* pNode = HitTestInList ( nX , nY );

	return pNode;
}


CModuleNode* CModuleGraph::HitTestInBFS ( CModuleNode* pNode , int& nX , int& nY )
{
	int i = 0;
	for ( i = 0 ; i < pNode->m_nChildNodeCount ; i++ )
	{
		if ( pNode->m_ppChildNodes[i]->PtInRect ( nX , nY ) )
		{
			return pNode->m_ppChildNodes[i];
		}
	}

	CModuleNode* pSearchNode = NULL;
	for ( i = 0 ; i < pNode->m_nChildNodeCount ; i++ )
	{
		pSearchNode = HitTestInBFS ( pNode->m_ppChildNodes[i] , nX , nY );
		if ( pSearchNode ) 
		{
			return pSearchNode;
		}
	}
	return NULL;
}

CModuleNode* CModuleGraph::HitTestInDFS ( CModuleNode* pNode , int& nX , int& nY )
{
	if ( pNode->PtInRect ( nX , nY ) )
	{
		return pNode;
	}

	int i = 0;
	CModuleNode* pSearchNode = NULL;
	for ( i = 0 ; i < pNode->m_nChildNodeCount ; i++ )
	{
		pSearchNode = HitTestInDFS ( pNode->m_ppChildNodes[i] , nX , nY );
		if ( pSearchNode ) 
		{
			return pSearchNode;
		}
	}
	return NULL;
}

CModuleNode* CModuleGraph::HitTestInList ( int& nX , int& nY )
{
	CModuleNode*				pNode = NULL;
	MODULENODE_REVERSE_ITER		itEnd = m_listModuleNode.rend();

	for ( MODULENODE_REVERSE_ITER it = m_listModuleNode.rbegin() ; it != itEnd ; ++it )
	{
		pNode = (*it);
		if ( pNode->PtInRect ( nX , nY ) )
		{
			return pNode;
		}
	}
	return NULL;
}

bool CModuleGraph::PtInPointWithMargin ( int& nX , int& nY , int& nInX , int& nInY )
{
	if ( nInX - PIN_TRACKER_MARGIN <= nX && nX <= nInX + PIN_TRACKER_MARGIN )
	{
		if ( nInY - PIN_TRACKER_MARGIN <= nY && nY <= nInY + PIN_TRACKER_MARGIN )
		{
			return true;
		}
	}
	return false;
}

CModuleNode* CModuleGraph::PinHitTest ( int nX , int nY , bool& bPinIn )
{
	CModuleNode*		pNode = NULL;
	MODULENODE_ITER		itEnd = m_listModuleNode.end();
	
	int nInOutX;
	int nInOutY;

	for ( MODULENODE_ITER it = m_listModuleNode.begin() ; it != itEnd ; ++it )
	{
		pNode = (*it);
		pNode->GetInputPoint ( nInOutX , nInOutY );
		if ( PtInPointWithMargin ( nX , nY , nInOutX , nInOutY ) )
		{
			bPinIn = true;
			return pNode;
		}

		pNode->GetOutputPoint ( nInOutX , nInOutY );
		if ( PtInPointWithMargin ( nX , nY , nInOutX , nInOutY ) )
		{
			bPinIn = false;
			return pNode;
		}
	}
	return NULL;
}


bool CModuleGraph::ConnectionHitTest ( int nX , int nY , CModuleNode*& pParentNode ,  CModuleNode*& pChildNode )
{
	CModuleNode*		pNode = NULL;
	MODULENODE_ITER		itEnd = m_listModuleNode.end();
	
	int nInX = 0;
	int nInY = 0;

	int nOutX = 0;
	int nOutY = 0;

	int i = 0;

	pParentNode	= NULL;
	pChildNode	= NULL;
	for ( MODULENODE_ITER it = m_listModuleNode.begin() ; it != itEnd ; ++it )
	{
		pNode = (*it);
		pNode->GetOutputPoint ( nOutX , nOutY );
		for ( i = 0 ; i < pNode->m_nChildNodeCount ; i++ )
		{
			pNode->m_ppChildNodes[i]->GetInputPoint ( nInX , nInY );
			if ( PtinLine ( (double)nOutX , (double)nOutY , (double)nInX , (double)nInY , 
														(double)nX , (double)nY , 4.0f ) )
			{
				pParentNode = pNode;
				pChildNode = pNode->m_ppChildNodes[i];
				return true;
			}
		}
	}
	return false;
}

void CModuleGraph::RemoveNodeInGraph ( CModuleNode* pNode )
{
	if ( pNode->m_pParentNode )
	{
		pNode->m_pParentNode->RemoveChildNode ( pNode );
	}

	for ( int i = 0 ; i < pNode->m_nChildNodeCount ; i++ )
	{
		pNode->m_ppChildNodes[i]->m_pParentNode = NULL;
	}
}

void CModuleGraph::RemoveNodeInList ( CModuleNode* pNode )
{
	CModuleNode*		pRemoveNode = NULL;
	MODULENODE_ITER		itEnd = m_listModuleNode.end();

	RemoveNodeInGraph ( pNode );

	for ( MODULENODE_ITER it = m_listModuleNode.begin() ; it != itEnd ; ++it )
	{
		pRemoveNode = (*it);
		if ( pRemoveNode == pNode )
		{
			/*
			if ( pRemoveNode->m_pParentNode )
			{
				pRemoveNode->m_pParentNode->RemoveChildNode ( pRemoveNode );
			}
*/
			delete pRemoveNode;
			m_listModuleNode.erase ( it );
			return;
		}
	}	
}

bool CModuleGraph::IsExistNode ( CModuleInfo* pInfo )
{
	CModuleNode*		pNode	= NULL;
	MODULENODE_ITER		itEnd	= m_listModuleNode.end();

	for ( MODULENODE_ITER it = m_listModuleNode.begin() ; it != itEnd ; ++it )
	{
		pNode = (*it);
		if ( pNode->m_pInfo == pInfo )
		{
			return true;
		}
	}	
	return false;
}

void CModuleGraph::InsertModule ( CModuleInfo* pInfo )
{
	if ( IsExistNode ( pInfo ) ) return;

	CModuleNode* pNode = new CModuleNode;
	pNode->m_pInfo = pInfo;

	m_listModuleNode.push_back ( pNode );
}

bool CModuleGraph::ConnectModule ( unsigned int uiParentModuleId , unsigned int uiChildModuleId )
{
	CModuleNode* pParentNode = NULL;
	CModuleNode* pChildNode = NULL;

	if ( (pParentNode = FindNodeInList ( uiParentModuleId )) == NULL )
	{
		return false;
	}

	if ( (pChildNode = FindNodeInList ( uiChildModuleId )) == NULL )
	{
		return false;
	}

	return ConnectModule ( pParentNode , pChildNode );
}

bool CModuleGraph::ConnectModule ( CModuleNode* pParentNode , CModuleNode* pChildNode )
{
	if ( NULL != pChildNode->m_pParentNode )
	{
		printf ( "ERROR : Pointer Address of ParentNode is NOT NULL\n" );
		return false;
	}
	if ( pParentNode->m_pInfo->m_uiOutputId == pChildNode->m_pInfo->m_uiInputId )
	{
		pParentNode->AddChildNode ( pChildNode );
		pChildNode->m_pParentNode = pParentNode;
	}
	else
	{
		return false;
	}
	return true;
}

void CModuleGraph::DisconnectModule ( unsigned int uiParentModuleId , unsigned int uiChildModuleId )
{
	CModuleNode* pParentNode = NULL;
	CModuleNode* pChildNode = NULL;

	if ( (pParentNode = FindNodeInList ( uiParentModuleId )) == NULL )
	{
		return;
	}
	
	if ( (pChildNode = pParentNode->FindInChildNodes ( uiChildModuleId )) == NULL )
	{
		return;
	}

	DisconnectModule ( pParentNode , pChildNode );
}

void CModuleGraph::DisconnectModule ( CModuleNode* pParentNode , CModuleNode* pChildNode )
{
	pParentNode->RemoveChildNode ( pChildNode );
	pChildNode->m_pParentNode = NULL;
}

void CModuleGraph::Initialize ( CAnalysisNotifyInterface* pInterface )
{
	m_pNotifyInterface = pInterface;
}

void CModuleGraph::StartAnalysisProcess ( CSourceFilePin* pSourceFilePin )
{
	CModuleNode* pNode = NULL;

	MODULENODE_ITER it;
	MODULENODE_ITER itEnd = m_listModuleNode.end();
	
	for ( it = m_listModuleNode.begin() ; it != itEnd ; ++it )
	{
		pNode = (*it);
		if ( pNode )
		{
			pNode->m_pInfo->Initialize ( m_pNotifyInterface , this );
		}
	}

	for ( it = m_listModuleNode.begin() ; it != itEnd ; ++it )
	{
		pNode = (*it);
		if ( IOP_SOURCE_FILES == pNode->m_pInfo->m_uiInputId )
		{
			// printf( "Start process begin\n" );
			pNode->m_pInfo->StartProcess ( pSourceFilePin );
		}
	}
}

void CModuleGraph::StopAnalysisProcess()
{
	CModuleNode* pNode = NULL;

	MODULENODE_ITER it;
	MODULENODE_ITER itEnd = m_listModuleNode.end();
	
	for ( it = m_listModuleNode.begin() ; it != itEnd ; ++it )
	{
		pNode = (*it);
		if ( pNode )
		{
			pNode->m_pInfo->StopProcess();
		}
	}
}

void CModuleGraph::OnRelayResult ( unsigned int uiModuleId , CInOutPinBase* pOutPin )
{
	CModuleNode* pNode = FindNodeInList ( uiModuleId );
	int nCount = pNode->m_nChildNodeCount;
	for ( int i = 0 ; i < nCount ; i++ )
	{
		pNode->m_ppChildNodes[i]->m_pInfo->StartProcess ( pOutPin );
	}
}
