#include "SourceTextView.h"
#include <sstream>
using namespace std;
#include "Main.h"
#include "MainFrame.h"
#include <iconv.h>

enum EParsingTextState
{
	PTS_NONE = 0 ,
	PTS_KEYWORD,
	PTS_KEYWORD_PREP,
	PTS_STRING,
	PTS_STRING_INCLUDE,
	PTS_COMMENT_LINE,
	PTS_COMMENT_RANGE,
	PTS_LETTER,
	PTS_NUMBER,
};
enum EKeywordPrepState
{
	KPS_NONE = 0 ,
	KPS_INCLUDE,
	KPS_IF_ELIF,
	KPS_PRAGMA,
	KPS_OTHERS,
};
CSourceTextView::CSourceTextView()
{
	//QueryPerformanceFrequency( &m_liSystemFrequency );	// TEST CODE

	m_pTextBuffer			=	m_tvwViewManager.get_buffer();

	RefPtr< Pango::Layout > pLayout = Pango::Layout::create( m_tvwViewManager.get_pango_context() );	// ̾ƿ 
	pLayout->set_text( "    " );
	pLayout->set_font_description( Pango::FontDescription( "Monospace 10" ) );
	Pango::TabArray TabArray( 1 );																//   
	TabArray.set_tab( 0, Pango::TAB_LEFT, pLayout->get_pixel_logical_extents().get_width() );
	m_tvwViewManager.set_tabs( TabArray );							//  ũ 
	m_tvwViewManager.modify_font( pLayout->get_font_description() );	// Ʈ 
	m_tvwViewManager.set_editable( false );							//  Ұ

	_InitializeKeyword();
	_InitializeKeywordPrep();
	_InitializeKeywordIfElif();
	_InitializeKeywordPragma();
	_InitializeTag();
	set_policy( Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC );
	m_pTextBuffer->signal_changed().connect( sigc::mem_fun( *this, &CSourceTextView::OnChangedTextBuffer ) );
	add( m_tvwViewManager );
	show_all_children();
}
CSourceTextView::~CSourceTextView()
{

}
bool CSourceTextView::SetFile( RefPtr< File > pFile, bool bDialog )
{
	m_pFile = pFile;
	if ( m_pFile->query_exists() == false )
	{
		if ( bDialog == true )
		{
			MessageDialog msgDialog( "The file is invalidated.\nPlease check state of the file." );
			msgDialog.set_title( "Error" );
			msgDialog.set_transient_for( *g_pmfMainFrame );
			msgDialog.run();
		}
		return false;
	}
	char szBuffer[ BUFFER_SIZE + 1 ];
	gssize sizeRead;
	string strBuffer;
	strBuffer.reserve( BUFFER_SIZE + 1 );
	RefPtr< FileInputStream > pfisStream = pFile->read();	// RefPtr ̸ ޾Ƴƾ read Ҷ  Ŀ ڵ ̵
	do
	{
		sizeRead = pfisStream->read( szBuffer, BUFFER_SIZE );
		szBuffer[ sizeRead ] = '\0';
		strBuffer += szBuffer;
	}
	while ( sizeRead == BUFFER_SIZE );

	ustring strUTF8;
	try{
		strUTF8 = locale_to_utf8( strBuffer );
	}
	catch(Glib::Error err )
	{
		strUTF8 = "";
		char* szOut = new char[ strBuffer.size()*3 +1 ];
		int nIn = 0, nOut = 0;
		char* pszIn = (char*)strBuffer.c_str();
		char* pszOut = szOut;
		memset( szOut, 0, strBuffer.size()*3 +1);
		nIn = strBuffer.size()+1;
		nOut = strBuffer.size()*3 + 1;

		iconv_t cd = iconv_open("UTF-8", "EUC-KR");
		iconv(cd, &pszIn, (size_t*)&nIn, &pszOut, (size_t*)&nOut);

		cout << nIn << endl;
		cout << szOut << endl;
		cout << nOut << endl;

		iconv_close(cd);
		

		m_pTextBuffer->set_text( szOut  );// );
		delete [] szOut;
	}
	
	if ( strUTF8.empty() == false )
	{
		m_pTextBuffer->set_text( strUTF8  );// );
	}
	return true;
}
RefPtr< File > CSourceTextView::GetFile()
{
	return m_pFile;
}
void CSourceTextView::Refresh()
{
	SetFile( m_pFile );
}
void CSourceTextView::OnChangedTextBuffer()
{
	//LARGE_INTEGER liStartTime;					// TEST CODE
	//LARGE_INTEGER liEndTime;					// TEST CODE
	//QueryPerformanceCounter( &liStartTime );	// TEST CODE

	_ChangeTextColor();
	//QueryPerformanceCounter( &liEndTime );		// TEST CODE
	//cout << ( liEndTime.QuadPart - liStartTime.QuadPart )/double( m_liSystemFrequency.QuadPart ) << endl;
}
void CSourceTextView::_InitializeKeyword()
{
	m_vecKeyword.push_back( "auto" );
	m_vecKeyword.push_back( "bool" );
	m_vecKeyword.push_back( "break" );
	m_vecKeyword.push_back( "case" );
	m_vecKeyword.push_back( "catch" );
	m_vecKeyword.push_back( "char" );
	m_vecKeyword.push_back( "class" );
	m_vecKeyword.push_back( "const" );
	m_vecKeyword.push_back( "const_cast" );
	m_vecKeyword.push_back( "continue" );
	m_vecKeyword.push_back( "default" );
	m_vecKeyword.push_back( "delete" );
	m_vecKeyword.push_back( "do" );
	m_vecKeyword.push_back( "double" );
	m_vecKeyword.push_back( "dynamic_cast" );
	m_vecKeyword.push_back( "else" );
	m_vecKeyword.push_back( "enum" );
	m_vecKeyword.push_back( "explicit" );
	m_vecKeyword.push_back( "extern" );
	m_vecKeyword.push_back( "false" );
	m_vecKeyword.push_back( "float" );
	m_vecKeyword.push_back( "for" );
	m_vecKeyword.push_back( "friend" );
	m_vecKeyword.push_back( "goto" );
	m_vecKeyword.push_back( "if" );
	m_vecKeyword.push_back( "inline" );
	m_vecKeyword.push_back( "int" );
	m_vecKeyword.push_back( "long" );
	m_vecKeyword.push_back( "mutable" );
	m_vecKeyword.push_back( "namespace" );
	m_vecKeyword.push_back( "new" );
	m_vecKeyword.push_back( "operator" );
	m_vecKeyword.push_back( "private" );
	m_vecKeyword.push_back( "protected" );
	m_vecKeyword.push_back( "public" );
	m_vecKeyword.push_back( "register" );
	m_vecKeyword.push_back( "reinterpret_cast" );
	m_vecKeyword.push_back( "return" );
	m_vecKeyword.push_back( "short" );
	m_vecKeyword.push_back( "signed" );
	m_vecKeyword.push_back( "sizeof" );
	m_vecKeyword.push_back( "static" );
	m_vecKeyword.push_back( "static_cast" );
	m_vecKeyword.push_back( "struct" );
	m_vecKeyword.push_back( "switch" );
	m_vecKeyword.push_back( "template" );
	m_vecKeyword.push_back( "this" );
	m_vecKeyword.push_back( "throw" );
	m_vecKeyword.push_back( "true" );
	m_vecKeyword.push_back( "try" );
	m_vecKeyword.push_back( "typedef" );
	m_vecKeyword.push_back( "typeid" );
	m_vecKeyword.push_back( "typename" );
	m_vecKeyword.push_back( "union" );
	m_vecKeyword.push_back( "unsigned" );
	m_vecKeyword.push_back( "using" );
	m_vecKeyword.push_back( "void" );
	m_vecKeyword.push_back( "volatile" );
	m_vecKeyword.push_back( "virtual" );
	m_vecKeyword.push_back( "wchar_t" );
	m_vecKeyword.push_back( "while" );
	m_vecKeyword.push_back( "__abstract" );
	m_vecKeyword.push_back( "__alignof" );
	m_vecKeyword.push_back( "__asm" );
	m_vecKeyword.push_back( "__assume" );
	m_vecKeyword.push_back( "__based" );
	m_vecKeyword.push_back( "__box" );
	m_vecKeyword.push_back( "__cdecl" );
	m_vecKeyword.push_back( "__declspec" );
	m_vecKeyword.push_back( "__delegate" );
	m_vecKeyword.push_back( "__event" );
	m_vecKeyword.push_back( "__except" );
	m_vecKeyword.push_back( "__fastcall" );
	m_vecKeyword.push_back( "__finally" );
	m_vecKeyword.push_back( "__forceinline" );
	m_vecKeyword.push_back( "__gc" );
	m_vecKeyword.push_back( "__hook" );
	m_vecKeyword.push_back( "__identifier" );
	m_vecKeyword.push_back( "__if_exists" );
	m_vecKeyword.push_back( "__if_not_exists" );
	m_vecKeyword.push_back( "__inline" );
	m_vecKeyword.push_back( "__int8" );
	m_vecKeyword.push_back( "__int16" );
	m_vecKeyword.push_back( "__int32" );
	m_vecKeyword.push_back( "__int64" );
	m_vecKeyword.push_back( "__interface" );
	m_vecKeyword.push_back( "__leave" );
	m_vecKeyword.push_back( "__m64" );
	m_vecKeyword.push_back( "__m128" );
	m_vecKeyword.push_back( "__m128d" );
	m_vecKeyword.push_back( "__m128i" );
	m_vecKeyword.push_back( "__multiple_inheritance" );
	m_vecKeyword.push_back( "__nogc" );
	m_vecKeyword.push_back( "__noop" );
	m_vecKeyword.push_back( "__pinv" );
	m_vecKeyword.push_back( "__property" );
	m_vecKeyword.push_back( "__raise" );
	m_vecKeyword.push_back( "__sealed" );
	m_vecKeyword.push_back( "__single_inheritance" );
	m_vecKeyword.push_back( "__stdcall" );
	m_vecKeyword.push_back( "__super" );
	m_vecKeyword.push_back( "__try_cast" );
	m_vecKeyword.push_back( "__try" );
	m_vecKeyword.push_back( "__except" );
	m_vecKeyword.push_back( "__finally" );
	m_vecKeyword.push_back( "__unhook" );
	m_vecKeyword.push_back( "__uuidof" );
	m_vecKeyword.push_back( "__value" );
	m_vecKeyword.push_back( "__virtual_inheritance" );
	m_vecKeyword.push_back( "__w64" );
	m_vecKeyword.push_back( "__wchar_t" );
	sort( m_vecKeyword.begin(), m_vecKeyword.end() );
	_GetMinMaxSize( m_vecKeyword, m_sizeKeywordMin, m_sizeKeywordMax );
}
void CSourceTextView::_InitializeKeywordPrep()
{
	m_vecKeywordPrep.push_back( "define" );
	m_vecKeywordPrep.push_back( "elif" );
	m_vecKeywordPrep.push_back( "else" );
	m_vecKeywordPrep.push_back( "endif" );
	m_vecKeywordPrep.push_back( "error" );
	m_vecKeywordPrep.push_back( "if" );
	m_vecKeywordPrep.push_back( "ifdef" );
	m_vecKeywordPrep.push_back( "ifndef" );
	m_vecKeywordPrep.push_back( "include" );
	m_vecKeywordPrep.push_back( "line" );
	m_vecKeywordPrep.push_back( "pragma" );
	m_vecKeywordPrep.push_back( "undef" );
	sort( m_vecKeywordPrep.begin(), m_vecKeywordPrep.end() );
	_GetMinMaxSize( m_vecKeywordPrep, m_sizeKeywordPrepMin, m_sizeKeywordPrepMax );
}
void CSourceTextView::_InitializeKeywordIfElif()
{
	m_vecKeywordIfElif.push_back( "defined" );
	sort( m_vecKeywordIfElif.begin(), m_vecKeywordIfElif.end() );
	_GetMinMaxSize( m_vecKeywordIfElif, m_sizeKeywordIfElifMin, m_sizeKeywordIfElifMax );
}
void CSourceTextView::_InitializeKeywordPragma()
{
	m_vecKeywordPragma.push_back( "alloc_text" );
	m_vecKeywordPragma.push_back( "comment" );
	m_vecKeywordPragma.push_back( "init_seg" );
	m_vecKeywordPragma.push_back( "optimize" );
	m_vecKeywordPragma.push_back( "auto_inline" );
	m_vecKeywordPragma.push_back( "component" );
	m_vecKeywordPragma.push_back( "inline_depth" );
	m_vecKeywordPragma.push_back( "pack" );
	m_vecKeywordPragma.push_back( "push" );
	m_vecKeywordPragma.push_back( "pop" );
	m_vecKeywordPragma.push_back( "bss_seg" );
	m_vecKeywordPragma.push_back( "data_seg" );
	m_vecKeywordPragma.push_back( "inline_recursion" );
	m_vecKeywordPragma.push_back( "pointers_to_members" );
	m_vecKeywordPragma.push_back( "check_stack" );
	m_vecKeywordPragma.push_back( "function" );
	m_vecKeywordPragma.push_back( "intrinsic" );
	m_vecKeywordPragma.push_back( "setlocale" );
	m_vecKeywordPragma.push_back( "code_seg" );
	m_vecKeywordPragma.push_back( "hdrstop" );
	m_vecKeywordPragma.push_back( "message" );
	m_vecKeywordPragma.push_back( "vtordisp" );
	m_vecKeywordPragma.push_back( "const_seg" );
	m_vecKeywordPragma.push_back( "include_alias" );
	m_vecKeywordPragma.push_back( "once" );
	m_vecKeywordPragma.push_back( "warning" );
	m_vecKeywordPragma.push_back( "disable" );
	m_vecKeywordPragma.push_back( "error" );
	m_vecKeywordPragma.push_back( "default" );
	m_vecKeywordPragma.push_back( "level" );
	m_vecKeywordPragma.push_back( "lib" );
	m_vecKeywordPragma.push_back( "exestr" );
	m_vecKeywordPragma.push_back( "user" );
	sort( m_vecKeywordPragma.begin(), m_vecKeywordPragma.end() );
	_GetMinMaxSize( m_vecKeywordPragma, m_sizeKeywordPragmaMin, m_sizeKeywordPragmaMax );
}
void CSourceTextView::_GetMinMaxSize( const vector< string >& vecString, size_t& sizeMin, size_t& sizeMax ) const
{
	if ( vecString.empty() == true )
	{
		sizeMin	=	0;
		sizeMax	=	0;
		return;
	}
	vector< string >::const_iterator itStringBegin	=	vecString.begin();
	vector< string >::const_iterator itStringEnd	=	vecString.end();
	vector< size_t > vecSize;
	for ( ; itStringBegin != itStringEnd ; ++itStringBegin )
	{
		vecSize.push_back( itStringBegin->size() );
	}
	sort( vecSize.begin(), vecSize.end() );
	sizeMin	=	*( vecSize.begin() );
	sizeMax	=	*( vecSize.end() - 1 );
}
void CSourceTextView::_InitializeTag()
{
	/*
	Color crTagColor;
	crTagColor.set_rgb_p( GDK_RGB( 255, 255, 255) ); m_tvwViewManager.modify_text( STATE_NORMAL, crTagColor );
	crTagColor.set_rgb_p( GDK_RGB(   0,   0,   0) ); m_tvwViewManager.modify_base( STATE_NORMAL, crTagColor );
	crTagColor.set_rgb_p( GDK_RGB( 255,   0,   0) ); m_tvwViewManager.modify_cursor( crTagColor, crTagColor );
	m_tvwViewManager.modify_base( STATE_SELECTED, crTagColor );
	*/
	Color crTagColor;
	crTagColor.set_rgb_p( GDK_RGB(   0,   0, 255) ); m_pTextBuffer->create_tag( "Keyword" )->property_foreground_gdk().set_value( crTagColor );
	crTagColor.set_rgb_p( GDK_RGB( 255,   0, 255) ); m_pTextBuffer->create_tag( "Number"  )->property_foreground_gdk().set_value( crTagColor );
	crTagColor.set_rgb_p( GDK_RGB( 255,   0, 255) ); m_pTextBuffer->create_tag( "Letter"  )->property_foreground_gdk().set_value( crTagColor );
	crTagColor.set_rgb_p( GDK_RGB( 222,  21,  21) ); m_pTextBuffer->create_tag( "String"  )->property_foreground_gdk().set_value( crTagColor );
	crTagColor.set_rgb_p( GDK_RGB(   0, 180,   0) ); m_pTextBuffer->create_tag( "Comment" )->property_foreground_gdk().set_value( crTagColor );
}
bool CSourceTextView::_IsSeparator( gunichar chLetter ) const
{
	if ( chLetter > 127 ||
		 chLetter == '_' ||
		 _IsNumber( chLetter ) == true ||
		 _IsAlphabet( chLetter ) == true )	// Ȯڵ, A-Z, a-z, 0-9, _  ڰ ƴ
	{
		return false;
	}
	return true;
}
bool CSourceTextView::_IsNumber( gunichar chLetter ) const
{
	if ( chLetter >= '0' && chLetter <= '9' )
	{
		return true;
	}
	return false;
}
bool CSourceTextView::_IsAlphabet( gunichar chLetter ) const
{
	if ( chLetter >= 'A' && chLetter <= 'Z' ||
		 chLetter >= 'a' && chLetter <= 'z' )
	{
		return true;
	}
	return false;
}
bool CSourceTextView::_IsKeyword( const string& strKeyword ) const
{
	size_t	sizeKeyword	=	strKeyword.size();
	if ( sizeKeyword < m_sizeKeywordMin || sizeKeyword > m_sizeKeywordMax )
	{
		return false;
	}
	return binary_search( m_vecKeyword.begin(), m_vecKeyword.end(), strKeyword );
}
bool CSourceTextView::_IsKeywordPrep( const string& strKeyword ) const
{
	size_t	sizeKeyword	=	strKeyword.size();
	if ( sizeKeyword < m_sizeKeywordPrepMin || sizeKeyword > m_sizeKeywordPrepMax )
	{
		return false;
	}
	return binary_search( m_vecKeywordPrep.begin(), m_vecKeywordPrep.end(), strKeyword );
}
bool CSourceTextView::_IsKeywordIfElif( const string& strKeyword ) const
{
	size_t	sizeKeyword	=	strKeyword.size();
	if ( sizeKeyword < m_sizeKeywordIfElifMin || sizeKeyword > m_sizeKeywordIfElifMax )
	{
		return false;
	}
	return binary_search( m_vecKeywordIfElif.begin(), m_vecKeywordIfElif.end(), strKeyword );
}
bool CSourceTextView::_IsKeywordPragma( const string& strKeyword ) const
{
	size_t	sizeKeyword	=	strKeyword.size();
	if ( sizeKeyword < m_sizeKeywordPragmaMin || sizeKeyword > m_sizeKeywordPragmaMax )
	{
		return false;
	}
	return binary_search( m_vecKeywordPragma.begin(), m_vecKeywordPragma.end(), strKeyword );
}
void CSourceTextView::_ChangeTextColor()
{
	m_pTextBuffer->remove_all_tags( m_pTextBuffer->begin(), m_pTextBuffer->end() );
	gunichar				chLetter;
	gunichar				chLetterNext;
	string					strKeyword;
	size_t					sizeBackSlashCount;
	EParsingTextState		enumParsingTextState	=	PTS_NONE;
	EKeywordPrepState		enumKeywordPrepState	=	KPS_NONE;
	TextBuffer::iterator	itTextBuffer			=	m_pTextBuffer->begin();
	TextBuffer::iterator	itTextBufferNext;
	TextBuffer::iterator	itTagStart;
	TextBuffer::iterator	itTagKeywordPerp;
	while ( 1 )
	{
		itTextBufferNext	=	itTextBuffer;
		++itTextBufferNext;
		chLetter			=	*itTextBuffer;
		chLetterNext		=	*itTextBufferNext;
		switch ( enumParsingTextState )
		{
		case PTS_NONE:
			{
				if ( chLetter == '\0' )
				{
					return;
				}
				else if ( chLetter == '/' && chLetterNext == '/')	// "//"
				{
					enumParsingTextState	=	PTS_COMMENT_LINE;
					itTagStart				=	itTextBuffer;
					++itTextBuffer;
					++itTextBuffer;
				}
				else if ( chLetter == '/' && chLetterNext == '*' )	// "/*"
				{
					enumParsingTextState	=	PTS_COMMENT_RANGE;
					itTagStart				=	itTextBuffer;
					++itTextBuffer;
					++itTextBuffer;
				}
				else if ( chLetter == '\"' )
				{
					enumParsingTextState	=	PTS_STRING;
					itTagStart				=	itTextBuffer;
					++itTextBuffer;
					sizeBackSlashCount		=	0;
				}
				else if ( chLetter == '\'' )
				{
					enumParsingTextState	=	PTS_LETTER;
					itTagStart				=	itTextBuffer;
					++itTextBuffer;
					sizeBackSlashCount		=	0;
				}
				else if ( _IsNumber( chLetter ) == true )			// ڷ Ѵٸ
				{
					enumParsingTextState	=	PTS_NUMBER;
					itTagStart				=	itTextBuffer;
					++itTextBuffer;
				}
				else if ( enumKeywordPrepState == KPS_NONE && chLetter == '#' )		// ó Ű ȣ Ѵٸ
				{
					enumParsingTextState	=	PTS_KEYWORD_PREP;
					itTagStart				=	itTextBuffer;
					++itTextBuffer;
					while ( *itTextBuffer == ' ' || *itTextBuffer == '\t' )
					{
						++itTextBuffer;
					}
					itTagKeywordPerp		=	itTextBuffer;
				}
				else if ( enumKeywordPrepState != KPS_NONE && chLetter == '\n' )	//  γ ó Ű尡 Ǿְ ٹٲ Ͼٸ
				{
					enumKeywordPrepState	=	KPS_NONE;
				}
				else if ( enumKeywordPrepState == KPS_INCLUDE && chLetter == '<' )
				{
					enumParsingTextState	=	PTS_STRING_INCLUDE;
					itTagStart				=	itTextBuffer;
					++itTextBuffer;
				}
				else if ( _IsSeparator( chLetter ) == false )	// ڰ ƴ Ϲ ڶ
				{
					enumParsingTextState	=	PTS_KEYWORD;
					itTagStart				=	itTextBuffer;
				}
				else											// Ÿ ڶ
				{
					++itTextBuffer;
				}
			}
			break;
		case PTS_KEYWORD:
			{
				if ( _IsSeparator( chLetterNext ) == true )
				{
					strKeyword = itTagStart.get_text( itTextBufferNext );
					if ( _IsKeyword( strKeyword ) == true ||
						enumKeywordPrepState == KPS_IF_ELIF && _IsKeywordIfElif( strKeyword ) == true ||
						enumKeywordPrepState == KPS_PRAGMA && _IsKeywordPragma( strKeyword ) == true )
					{
						m_pTextBuffer->apply_tag_by_name( "Keyword", itTagStart, itTextBufferNext );
					}
					if ( chLetterNext == '\0' )
					{
						return;
					}
					enumParsingTextState	=	PTS_NONE;
				}
				++itTextBuffer;
			}
			break;
		case PTS_KEYWORD_PREP:
			{
				if ( _IsSeparator( chLetterNext ) == true )
				{
					strKeyword = itTagKeywordPerp.get_text( itTextBufferNext );
					if ( _IsKeywordPrep( strKeyword ) == true )
					{
						m_pTextBuffer->apply_tag_by_name( "Keyword", itTagStart, itTextBufferNext );
						if ( strKeyword == "include" )
						{
							enumKeywordPrepState	=	KPS_INCLUDE;
						}
						else if ( strKeyword == "if" || strKeyword == "elif" )
						{
							enumKeywordPrepState	=	KPS_IF_ELIF;
						}
						else if ( strKeyword == "pragma" )
						{
							enumKeywordPrepState	=	KPS_PRAGMA;
						}
						else
						{
							enumKeywordPrepState	=	KPS_OTHERS;
						}
					}
					if ( chLetterNext == '\0' )
					{
						return;
					}
					enumParsingTextState	=	PTS_NONE;
				}
				++itTextBuffer;
			}
			break;
		case PTS_STRING:
			{
				if ( ( chLetter == '\"' || chLetter == '\n' ) && sizeBackSlashCount % 2 == 0 )
				{
					m_pTextBuffer->apply_tag_by_name( "String", itTagStart, itTextBufferNext );
					enumParsingTextState	=	PTS_NONE;
				}
				else if ( chLetterNext == '\0' )
				{
					m_pTextBuffer->apply_tag_by_name( "String", itTagStart, itTextBufferNext );
					return;
				}
				else if ( chLetter == '\\' )
				{
					++sizeBackSlashCount;
				}
				else
				{
					sizeBackSlashCount = 0;
				}
				++itTextBuffer;
			}
			break;
		case PTS_STRING_INCLUDE:
			{
				if ( chLetter == '>' || chLetter == '\n' )
				{
					m_pTextBuffer->apply_tag_by_name( "String", itTagStart, itTextBufferNext );
					enumParsingTextState	=	PTS_NONE;
					enumKeywordPrepState	=	KPS_NONE;
				}
				else if ( chLetterNext == '\0' )
				{
					m_pTextBuffer->apply_tag_by_name( "String", itTagStart, itTextBufferNext );
					return;
				}
				++itTextBuffer;
			}
			break;
		case PTS_COMMENT_LINE:
			{
				if ( chLetter != '\\' && chLetterNext == '\n' )
				{
					m_pTextBuffer->apply_tag_by_name( "Comment", itTagStart, itTextBufferNext );
					enumParsingTextState	=	PTS_NONE;
				}
				else if ( chLetterNext == '\0' )
				{
					m_pTextBuffer->apply_tag_by_name( "Comment", itTagStart, itTextBufferNext );
					return;
				}
				++itTextBuffer;
			}
			break;
		case PTS_COMMENT_RANGE:
			{
				if ( chLetter == '*' && chLetterNext == '/' )
				{
					++itTextBufferNext;
					m_pTextBuffer->apply_tag_by_name( "Comment", itTagStart, itTextBufferNext );
					enumParsingTextState	=	PTS_NONE;
					++itTextBuffer;
				}
				else if ( chLetterNext == '\0' )
				{
					++itTextBufferNext;
					m_pTextBuffer->apply_tag_by_name( "Comment", itTagStart, itTextBufferNext );
					return;
				}
				++itTextBuffer;
			}
			break;
		case PTS_LETTER:
			{
				if ( ( chLetter == '\'' || chLetter == '\n' ) && sizeBackSlashCount % 2 == 0 )
				{
					m_pTextBuffer->apply_tag_by_name( "String", itTagStart, itTextBufferNext );
					enumParsingTextState	=	PTS_NONE;
				}
				else if ( chLetterNext == '\0' )
				{
					m_pTextBuffer->apply_tag_by_name( "String", itTagStart, itTextBufferNext );
					return;
				}
				else if ( chLetter == '\\' )
				{
					++sizeBackSlashCount;
				}
				else
				{
					sizeBackSlashCount = 0;
				}
				++itTextBuffer;
			}
			break;
		case PTS_NUMBER:
			{
				if ( _IsNumber( chLetterNext ) == false && _IsAlphabet( chLetterNext ) == false && chLetterNext != '.' )
				{
					m_pTextBuffer->apply_tag_by_name( "Number", itTagStart, itTextBufferNext );
					if ( chLetterNext == '\0' )
					{
						return;
					}
					enumParsingTextState	=	PTS_NONE;
				}
				++itTextBuffer;
			}
			break;
		}
	}
}


